@@ -8,7 +8,6 @@ from functools import reduce
 from typing import Tuple, Union
 
 import opt_einsum
-from multipledispatch.variadic import Variadic
 
 import funsor
 import funsor.ops as ops
@@ -16,10 +15,12 @@ from funsor.affine import affine_inputs
 from funsor.delta import Delta
 from funsor.domains import find_domain
 from funsor.gaussian import Gaussian
-from funsor.interpreter import interpretation, recursion_reinterpret
-from funsor.ops import DISTRIBUTIVE_OPS, AssociativeOp, NullOp, nullop
+from funsor.interpretations import eager, normalize, reflect
+from funsor.interpreter import children
+from funsor.ops import DISTRIBUTIVE_OPS, AssociativeOp, NullOp
 from funsor.tensor import Tensor
 from funsor.terms import (
+    _INFIX,
     Align,
     Binary,
     Funsor,
@@ -28,11 +29,9 @@ from funsor.terms import (
     Subs,
     Unary,
     Variable,
-    eager,
-    normalize,
-    reflect,
-    to_funsor
+    to_funsor,
 )
+from funsor.typing import Variadic
 from funsor.util import broadcast_shape, get_backend, quote
 
 
@@ -45,6 +44,7 @@ class Contraction(Funsor):
 
         Delta, Number, Tensor, Gaussian
     """
+
     def __init__(self, red_op, bin_op, reduced_vars, terms):
         terms = (terms,) if isinstance(terms, Funsor) else terms
         assert isinstance(red_op, AssociativeOp)
@@ -69,105 +69,171 @@ class Contraction(Funsor):
         for v in terms:
             inputs.update((k, d) for k, d in v.inputs.items() if k not in bound)
 
-        if bin_op is nullop:
+        if bin_op is ops.null:
             output = terms[0].output
         else:
-            output = reduce(lambda lhs, rhs: find_domain(bin_op, lhs, rhs),
-                            [v.output for v in reversed(terms)])
+            output = reduce(
+                lambda lhs, rhs: find_domain(bin_op, lhs, rhs),
+                [v.output for v in reversed(terms)],
+            )
         super(Contraction, self).__init__(inputs, output, fresh, bound)
         self.red_op = red_op
         self.bin_op = bin_op
         self.terms = terms
         self.reduced_vars = reduced_vars
 
-    def unscaled_sample(self, sampled_vars, sample_inputs, rng_key=None):
+    def __repr__(self):
+        if self.bin_op in _INFIX:
+            bin_op = " " + _INFIX[self.bin_op] + " "
+            return "{}.reduce({}, {})".format(
+                bin_op.join(map(repr, self.terms)),
+                self.red_op,
+                str(set(self.reduced_vars)),
+            )
+        return super().__repr__()
+
+    def __str__(self):
+        if self.bin_op in _INFIX:
+            bin_op = " " + _INFIX[self.bin_op] + " "
+            return "({}).reduce({}, {})".format(
+                bin_op.join(map(str, self.terms)),
+                self.red_op,
+                str(set(map(str, self.reduced_vars))),
+            )
+        return super().__str__()
+
+    def _sample(self, sampled_vars, sample_inputs, rng_key):
         sampled_vars = sampled_vars.intersection(self.inputs)
         if not sampled_vars:
             return self
+        for term in self.terms:
+            if isinstance(term, Delta):
+                sampled_vars -= term.fresh
+        if not sampled_vars:
+            return self
 
-        if self.red_op in (ops.logaddexp, nullop):
-            if self.bin_op in (ops.nullop, ops.logaddexp):
-                if rng_key is not None and get_backend() == "jax":
-                    import jax
-                    rng_keys = jax.random.split(rng_key, len(self.terms))
-                else:
-                    rng_keys = [None] * len(self.terms)
+        if self.red_op in (ops.null, ops.logaddexp):
+            if rng_key is not None and get_backend() == "jax":
+                import jax
 
-                # Design choice: we sample over logaddexp reductions, but leave logaddexp
-                # binary choices symbolic.
+                rng_keys = jax.random.split(rng_key, len(self.terms))
+            else:
+                rng_keys = [None] * len(self.terms)
+
+            if self.bin_op in (ops.null, ops.logaddexp):
+                # Design choice: we sample over logaddexp reductions, but leave
+                # logaddexp binary choices symbolic.
                 terms = [
-                    term.unscaled_sample(sampled_vars.intersection(term.inputs), sample_inputs)
-                    for term, rng_key in zip(self.terms, rng_keys)]
+                    term._sample(
+                        sampled_vars.intersection(term.inputs), sample_inputs, rng_key
+                    )
+                    for term, rng_key in zip(self.terms, rng_keys)
+                ]
                 return Contraction(self.red_op, self.bin_op, self.reduced_vars, *terms)
 
             if self.bin_op is ops.add:
-                if rng_key is not None and get_backend() == "jax":
-                    import jax
-                    rng_keys = jax.random.split(rng_key)
-                else:
-                    rng_keys = [None] * 2
-
                 # Sample variables greedily in order of the terms in which they appear.
                 for term in self.terms:
                     greedy_vars = sampled_vars.intersection(term.inputs)
                     if greedy_vars:
                         break
+                assert greedy_vars
                 greedy_terms, terms = [], []
                 for term in self.terms:
-                    (terms if greedy_vars.isdisjoint(term.inputs) else greedy_terms).append(term)
+                    if greedy_vars.isdisjoint(term.inputs):
+                        terms.append(term)
+                    elif isinstance(term, Delta) and greedy_vars.isdisjoint(term.fresh):
+                        terms.append(term)
+                    else:
+                        greedy_terms.append(term)
                 if len(greedy_terms) == 1:
                     term = greedy_terms[0]
-                    terms.append(term.unscaled_sample(greedy_vars, sample_inputs, rng_keys[0]))
-                    result = Contraction(self.red_op, self.bin_op, self.reduced_vars, *terms)
-                elif (len(greedy_terms) == 2 and
-                        isinstance(greedy_terms[0], Tensor) and
-                        isinstance(greedy_terms[1], Gaussian)):
+                    terms.append(term._sample(greedy_vars, sample_inputs, rng_keys[0]))
+                    result = Contraction(
+                        self.red_op, self.bin_op, self.reduced_vars, *terms
+                    )
+                elif (
+                    len(greedy_terms) == 2
+                    and isinstance(greedy_terms[0], Tensor)
+                    and isinstance(greedy_terms[1], Gaussian)
+                ):
                     discrete, gaussian = greedy_terms
                     term = discrete + gaussian.log_normalizer
                     terms.append(gaussian)
                     terms.append(-gaussian.log_normalizer)
-                    terms.append(term.unscaled_sample(greedy_vars, sample_inputs, rng_keys[0]))
-                    result = Contraction(self.red_op, self.bin_op, self.reduced_vars, *terms)
-                elif any(isinstance(term, funsor.distribution.Distribution)
-                         and not greedy_vars.isdisjoint(term.value.inputs) for term in greedy_terms):
+                    terms.append(term._sample(greedy_vars, sample_inputs, rng_keys[0]))
+                    result = Contraction(
+                        self.red_op, self.bin_op, self.reduced_vars, *terms
+                    )
+                elif any(
+                    isinstance(term, funsor.distribution.Distribution)
+                    and not greedy_vars.isdisjoint(term.value.inputs)
+                    for term in greedy_terms
+                ):
                     sampled_terms = [
-                        term.unscaled_sample(greedy_vars.intersection(term.value.inputs), sample_inputs)
-                        for term in greedy_terms if isinstance(term, funsor.distribution.Distribution)
+                        term._sample(
+                            greedy_vars.intersection(term.value.inputs),
+                            sample_inputs,
+                            rng_key,
+                        )
+                        for term, rng_key in zip(greedy_terms, rng_keys)
+                        if isinstance(term, funsor.distribution.Distribution)
                         and not greedy_vars.isdisjoint(term.value.inputs)
                     ]
-                    result = Contraction(self.red_op, self.bin_op, self.reduced_vars, *(terms + sampled_terms))
+                    result = Contraction(
+                        self.red_op,
+                        self.bin_op,
+                        self.reduced_vars,
+                        *(terms + sampled_terms)
+                    )
                 else:
-                    raise NotImplementedError('Unhandled case: {}'.format(
-                        ', '.join(str(type(t)) for t in greedy_terms)))
-                return result.unscaled_sample(sampled_vars - greedy_vars, sample_inputs, rng_keys[1])
+                    raise NotImplementedError(
+                        "Unhandled case: {}".format(
+                            ", ".join(str(type(t)) for t in greedy_terms)
+                        )
+                    )
+                return result._sample(
+                    sampled_vars - greedy_vars, sample_inputs, rng_keys[1]
+                )
 
-        raise TypeError("Cannot sample through ops ({}, {})".format(self.red_op, self.bin_op))
+        raise TypeError(
+            "Cannot sample through ops ({}, {})".format(self.red_op, self.bin_op)
+        )
 
     def align(self, names):
         assert isinstance(names, tuple)
         assert all(name in self.inputs for name in names)
-        new_terms = tuple(t.align(tuple(n for n in names if n in t.inputs)) for t in self.terms)
+        new_terms = tuple(
+            t.align(tuple(n for n in names if n in t.inputs)) for t in self.terms
+        )
         result = Contraction(self.red_op, self.bin_op, self.reduced_vars, *new_terms)
         if not names == tuple(result.inputs):
-            return Align(result, names)  # raise NotImplementedError("TODO align all terms")
+            return Align(
+                result, names
+            )  # raise NotImplementedError("TODO align all terms")
         return result
 
     def _alpha_convert(self, alpha_subs):
-        reduced_vars = frozenset(to_funsor(alpha_subs.get(var.name, var), var.output)
-                                 for var in self.reduced_vars)
+        reduced_vars = frozenset(
+            to_funsor(alpha_subs.get(var.name, var), var.output)
+            for var in self.reduced_vars
+        )
         alpha_subs = {k: to_funsor(v, self.bound[k]) for k, v in alpha_subs.items()}
         red_op, bin_op, _, terms = super()._alpha_convert(alpha_subs)
         return red_op, bin_op, reduced_vars, terms
 
 
-GaussianMixture = Contraction[Union[ops.LogaddexpOp, NullOp], ops.AddOp, frozenset,
-                              Tuple[Union[Tensor, Number], Gaussian]]
+GaussianMixture = Contraction[
+    Union[ops.LogaddexpOp, NullOp],
+    ops.AddOp,
+    frozenset,
+    Tuple[Union[Tensor, Number], Gaussian],
+]
 
 
 @quote.register(Contraction)
 def _(arg, indent, out):
-    line = "{}({}, {},".format(
-        type(arg).__name__, repr(arg.red_op), repr(arg.bin_op))
+    line = "{}({}, {},".format(type(arg).__name__, repr(arg.red_op), repr(arg.bin_op))
     out.append((indent, line))
     quote.inplace(arg.reduced_vars, indent + 1, out)
     i, line = out[-1]
@@ -177,14 +243,19 @@ def _(arg, indent, out):
     out[-1] = i, line + ")"
 
 
-@recursion_reinterpret.register(Contraction)
-def recursion_reinterpret_contraction(x):
-    return type(x)(*map(recursion_reinterpret, (x.red_op, x.bin_op, x.reduced_vars) + x.terms))
+@children.register(Contraction)
+def children_contraction(x):
+    return (x.red_op, x.bin_op, x.reduced_vars) + x.terms
+
+
+@children.register(Contraction)
+def children_contraction(x):
+    return (x.red_op, x.bin_op, x.reduced_vars) + x.terms
 
 
 @eager.register(Contraction, AssociativeOp, AssociativeOp, frozenset, Variadic[Funsor])
 def eager_contraction_generic_to_tuple(red_op, bin_op, reduced_vars, *terms):
-    return eager(Contraction, red_op, bin_op, reduced_vars, terms)
+    return eager.interpret(Contraction, red_op, bin_op, reduced_vars, terms)
 
 
 @eager.register(Contraction, AssociativeOp, AssociativeOp, frozenset, tuple)
@@ -203,7 +274,9 @@ def eager_contraction_generic_recursive(red_op, bin_op, reduced_vars, terms):
             unique_vars = reduced_once & term.input_vars
             if unique_vars:
                 result = term.reduce(red_op, unique_vars)
-                if result is not normalize(Contraction, red_op, nullop, unique_vars, (term,)):
+                if result is not normalize.interpret(
+                    Contraction, red_op, ops.null, unique_vars, (term,)
+                ):
                     terms[i] = result
                     reduced_vars -= unique_vars
                     leaf_reduced = True
@@ -215,14 +288,16 @@ def eager_contraction_generic_recursive(red_op, bin_op, reduced_vars, terms):
     terms = tuple(terms)
     reduced_twice = frozenset(v for v, count in counts.items() if count == 2)
     for i, lhs in enumerate(terms[0:-1]):
-        for j_, rhs in enumerate(terms[i+1:]):
+        for j_, rhs in enumerate(terms[i + 1 :]):
             j = i + j_ + 1
             unique_vars = reduced_twice.intersection(lhs.input_vars, rhs.input_vars)
             result = Contraction(red_op, bin_op, unique_vars, lhs, rhs)
-            if result is not normalize(Contraction, red_op, bin_op, unique_vars, (lhs, rhs)):  # did we make progress?
+            if result is not normalize.interpret(
+                Contraction, red_op, bin_op, unique_vars, (lhs, rhs)
+            ):  # did we make progress?
                 # pick the first evaluable pair
                 reduced_vars -= unique_vars
-                new_terms = terms[:i] + (result,) + terms[i+1:j] + terms[j+1:]
+                new_terms = terms[:i] + (result,) + terms[i + 1 : j] + terms[j + 1 :]
                 return Contraction(red_op, bin_op, reduced_vars, *new_terms)
 
     return None
@@ -246,15 +321,14 @@ def eager_contraction_to_binary(red_op, bin_op, reduced_vars, lhs, rhs):
     args = bin_op, lhs, rhs
     result = eager.dispatch(Binary, *args)(*args)
     if result is not None and reduced_vars:
-        args = red_op, result, reduced_vars
-        result = eager.dispatch(Reduce, *args)(*args)
+        result = eager.interpret(Reduce, red_op, result, reduced_vars)
     return result
 
 
 @eager.register(Contraction, ops.AddOp, ops.MulOp, frozenset, Tensor, Tensor)
 def eager_contraction_tensor(red_op, bin_op, reduced_vars, *terms):
     if not all(term.dtype == "real" for term in terms):
-        raise NotImplementedError('TODO')
+        raise NotImplementedError("TODO")
     backend = BACKEND_TO_EINSUM_BACKEND[get_backend()]
     return _eager_contract_tensors(reduced_vars, terms, backend=backend)
 
@@ -262,7 +336,7 @@ def eager_contraction_tensor(red_op, bin_op, reduced_vars, *terms):
 @eager.register(Contraction, ops.LogaddexpOp, ops.AddOp, frozenset, Tensor, Tensor)
 def eager_contraction_tensor(red_op, bin_op, reduced_vars, *terms):
     if not all(term.dtype == "real" for term in terms):
-        raise NotImplementedError('TODO')
+        raise NotImplementedError("TODO")
     backend = BACKEND_TO_LOGSUMEXP_BACKEND[get_backend()]
     return _eager_contract_tensors(reduced_vars, terms, backend=backend)
 
@@ -277,14 +351,18 @@ def _eager_contract_tensors(reduced_vars, terms, backend):
     operands = []
     for term in terms:
         inputs.update(term.inputs)
-        einsum_inputs.append("".join(symbols[k] for k in term.inputs) +
-                             "".join(symbols[i - len(term.shape)]
-                                     for i, size in enumerate(term.shape)
-                                     if size != 1))
+        einsum_inputs.append(
+            "".join(symbols[k] for k in term.inputs)
+            + "".join(
+                symbols[i - len(term.shape)]
+                for i, size in enumerate(term.shape)
+                if size != 1
+            )
+        )
 
         # Squeeze absent event dims to be compatible with einsum.
         data = term.data
-        batch_shape = data.shape[:len(data.shape) - len(term.shape)]
+        batch_shape = data.shape[: len(data.shape) - len(term.shape)]
         event_shape = tuple(size for size in term.shape if size != 1)
         data = data.reshape(batch_shape + event_shape)
         operands.append(data)
@@ -293,10 +371,9 @@ def _eager_contract_tensors(reduced_vars, terms, backend):
         inputs.pop(var.name, None)
     batch_shape = tuple(v.size for v in inputs.values())
     event_shape = broadcast_shape(*(term.shape for term in terms))
-    einsum_output = ("".join(symbols[k] for k in inputs) +
-                     "".join(symbols[dim]
-                             for dim in range(-len(event_shape), 0)
-                             if dim in symbols))
+    einsum_output = "".join(symbols[k] for k in inputs) + "".join(
+        symbols[dim] for dim in range(-len(event_shape), 0) if dim in symbols
+    )
     equation = ",".join(einsum_inputs) + "->" + einsum_output
     data = opt_einsum.contract(equation, *operands, backend=backend)
     data = data.reshape(batch_shape + event_shape)
@@ -306,15 +383,16 @@ def _eager_contract_tensors(reduced_vars, terms, backend):
 # TODO(https://github.com/pyro-ppl/funsor/issues/238) Use a port of
 # Pyro's gaussian_tensordot() here. Until then we must eagerly add the
 # possibly-rank-deficient terms before reducing to avoid Cholesky errors.
-@eager.register(Contraction, ops.LogaddexpOp, ops.AddOp, frozenset,
-                GaussianMixture, GaussianMixture)
+@eager.register(
+    Contraction, ops.LogaddexpOp, ops.AddOp, frozenset, GaussianMixture, GaussianMixture
+)
 def eager_contraction_gaussian(red_op, bin_op, reduced_vars, x, y):
     return (x + y).reduce(red_op, reduced_vars)
 
 
 @affine_inputs.register(Contraction)
 def _(fn):
-    with interpretation(reflect):
+    with reflect:
         flat = reduce(fn.bin_op, fn.terms).reduce(fn.red_op, fn.reduced_vars)
     return affine_inputs(flat)
 
@@ -323,37 +401,58 @@ def _(fn):
 # Normalizing Contractions
 ##########################################
 
-ORDERING = {Delta: 1, Number: 2, Tensor: 3, Gaussian: 4}
+ORDERING = {Delta: 1, Number: 2, Tensor: 3, Gaussian: 4, Unary[ops.NegOp, Gaussian]: 5}
 GROUND_TERMS = tuple(ORDERING)
 
 
-@normalize.register(Contraction, AssociativeOp, ops.AddOp, frozenset, GROUND_TERMS, GROUND_TERMS)
-def normalize_contraction_commutative_canonical_order(red_op, bin_op, reduced_vars, *terms):
+@normalize.register(
+    Contraction, AssociativeOp, ops.AddOp, frozenset, GROUND_TERMS, GROUND_TERMS
+)
+def normalize_contraction_commutative_canonical_order(
+    red_op, bin_op, reduced_vars, *terms
+):
     # when bin_op is commutative, put terms into a canonical order for pattern matching
     new_terms = tuple(
-        v for i, v in sorted(enumerate(terms),
-                             key=lambda t: (ORDERING.get(type(t[1]).__origin__, -1), t[0]))
+        v
+        for i, v in sorted(
+            enumerate(terms),
+            key=lambda t: (ORDERING.get(type(t[1]).__origin__, -1), t[0]),
+        )
     )
     if any(v is not vv for v, vv in zip(terms, new_terms)):
         return Contraction(red_op, bin_op, reduced_vars, *new_terms)
-    return normalize(Contraction, red_op, bin_op, reduced_vars, new_terms)
+    return normalize.interpret(Contraction, red_op, bin_op, reduced_vars, new_terms)
 
 
-@normalize.register(Contraction, AssociativeOp, ops.AddOp, frozenset, GaussianMixture, GROUND_TERMS)
+@normalize.register(
+    Contraction, AssociativeOp, ops.AddOp, frozenset, GaussianMixture, GROUND_TERMS
+)
 def normalize_contraction_commute_joint(red_op, bin_op, reduced_vars, mixture, other):
-    return Contraction(mixture.red_op if red_op is nullop else red_op, bin_op,
-                       reduced_vars | mixture.reduced_vars, *(mixture.terms + (other,)))
+    return Contraction(
+        mixture.red_op if red_op is ops.null else red_op,
+        bin_op,
+        reduced_vars | mixture.reduced_vars,
+        *(mixture.terms + (other,))
+    )
 
 
-@normalize.register(Contraction, AssociativeOp, ops.AddOp, frozenset, GROUND_TERMS, GaussianMixture)
+@normalize.register(
+    Contraction, AssociativeOp, ops.AddOp, frozenset, GROUND_TERMS, GaussianMixture
+)
 def normalize_contraction_commute_joint(red_op, bin_op, reduced_vars, other, mixture):
-    return Contraction(mixture.red_op if red_op is nullop else red_op, bin_op,
-                       reduced_vars | mixture.reduced_vars, *(mixture.terms + (other,)))
+    return Contraction(
+        mixture.red_op if red_op is ops.null else red_op,
+        bin_op,
+        reduced_vars | mixture.reduced_vars,
+        *(mixture.terms + (other,))
+    )
 
 
-@normalize.register(Contraction, AssociativeOp, AssociativeOp, frozenset, Variadic[Funsor])
+@normalize.register(
+    Contraction, AssociativeOp, AssociativeOp, frozenset, Variadic[Funsor]
+)
 def normalize_contraction_generic_args(red_op, bin_op, reduced_vars, *terms):
-    return normalize(Contraction, red_op, bin_op, reduced_vars, tuple(terms))
+    return normalize.interpret(Contraction, red_op, bin_op, reduced_vars, tuple(terms))
 
 
 @normalize.register(Contraction, NullOp, NullOp, frozenset, Funsor)
@@ -365,21 +464,27 @@ def normalize_trivial(red_op, bin_op, reduced_vars, term):
 @normalize.register(Contraction, AssociativeOp, AssociativeOp, frozenset, tuple)
 def normalize_contraction_generic_tuple(red_op, bin_op, reduced_vars, terms):
 
-    if not reduced_vars and red_op is not nullop:
-        return Contraction(nullop, bin_op, reduced_vars, *terms)
+    if not reduced_vars and red_op is not ops.null:
+        return Contraction(ops.null, bin_op, reduced_vars, *terms)
 
-    if len(terms) == 1 and bin_op is not nullop:
-        return Contraction(red_op, nullop, reduced_vars, *terms)
+    if len(terms) == 1 and bin_op is not ops.null:
+        return Contraction(red_op, ops.null, reduced_vars, *terms)
 
-    if red_op is nullop and bin_op is nullop:
+    if red_op is ops.null and bin_op is ops.null:
         return terms[0]
 
     if red_op is bin_op:
         new_terms = tuple(v.reduce(red_op, reduced_vars) for v in terms)
         return Contraction(red_op, bin_op, frozenset(), *new_terms)
 
-    if bin_op in ops.UNITS and any(isinstance(t, Number) and t.data == ops.UNITS[bin_op] for t in terms):
-        new_terms = tuple(t for t in terms if not (isinstance(t, Number) and t.data == ops.UNITS[bin_op]))
+    if bin_op in ops.UNITS and any(
+        isinstance(t, Number) and t.data == ops.UNITS[bin_op] for t in terms
+    ):
+        new_terms = tuple(
+            t
+            for t in terms
+            if not (isinstance(t, Number) and t.data == ops.UNITS[bin_op])
+        )
         if not new_terms:  # everything was a unit
             new_terms = (terms[0],)
         return Contraction(red_op, bin_op, reduced_vars, *new_terms)
@@ -390,12 +495,15 @@ def normalize_contraction_generic_tuple(red_op, bin_op, reduced_vars, terms):
             continue
 
         # fuse operations without distributing
-        if (v.red_op is nullop and bin_op is v.bin_op) or \
-                (bin_op is nullop and v.red_op in (red_op, nullop)):
-            red_op = v.red_op if red_op is nullop else red_op
-            bin_op = v.bin_op if bin_op is nullop else bin_op
-            new_terms = terms[:i] + v.terms + terms[i+1:]
-            return Contraction(red_op, bin_op, reduced_vars | v.reduced_vars, *new_terms)
+        if (v.red_op is ops.null and bin_op is v.bin_op) or (
+            bin_op is ops.null and v.red_op in (red_op, ops.null)
+        ):
+            red_op = v.red_op if red_op is ops.null else red_op
+            bin_op = v.bin_op if bin_op is ops.null else bin_op
+            new_terms = terms[:i] + v.terms + terms[i + 1 :]
+            return Contraction(
+                red_op, bin_op, reduced_vars | v.reduced_vars, *new_terms
+            )
 
     # nothing more to do, reflect
     return None
@@ -405,17 +513,22 @@ def normalize_contraction_generic_tuple(red_op, bin_op, reduced_vars, terms):
 # Creating Contractions from other terms
 #########################################
 
+
 @normalize.register(Binary, AssociativeOp, Funsor, Funsor)
 def binary_to_contract(op, lhs, rhs):
-    return Contraction(nullop, op, frozenset(), lhs, rhs)
+    return Contraction(ops.null, op, frozenset(), lhs, rhs)
 
 
 @normalize.register(Reduce, AssociativeOp, Funsor, frozenset)
 def reduce_funsor(op, arg, reduced_vars):
-    return Contraction(op, nullop, reduced_vars, arg)
+    return Contraction(op, ops.null, reduced_vars, arg)
 
 
-@normalize.register(Unary, ops.NegOp, (Variable, Contraction[ops.AssociativeOp, ops.MulOp, frozenset, tuple]))
+@normalize.register(
+    Unary,
+    ops.NegOp,
+    (Variable, Contraction[ops.AssociativeOp, ops.MulOp, frozenset, tuple]),
+)
 def unary_neg_variable(op, arg):
     return arg * -1
 
@@ -424,6 +537,7 @@ def unary_neg_variable(op, arg):
 # Distributing Unary transformations (Subs, log, exp, neg, reciprocal)
 #######################################################################
 
+
 @normalize.register(Subs, Funsor, tuple)
 def do_fresh_subs(arg, subs):
     if not subs:
@@ -435,17 +549,21 @@ def do_fresh_subs(arg, subs):
 
 @normalize.register(Subs, Contraction, tuple)
 def distribute_subs_contraction(arg, subs):
-    new_terms = tuple(Subs(v, tuple((name, sub) for name, sub in subs if name in v.inputs))
-                      if any(name in v.inputs for name, sub in subs)
-                      else v
-                      for v in arg.terms)
+    new_terms = tuple(
+        Subs(v, tuple((name, sub) for name, sub in subs if name in v.inputs))
+        if any(name in v.inputs for name, sub in subs)
+        else v
+        for v in arg.terms
+    )
     return Contraction(arg.red_op, arg.bin_op, arg.reduced_vars, *new_terms)
 
 
 @normalize.register(Subs, Subs, tuple)
 def normalize_fuse_subs(arg, subs):
     # a(b)(c) -> a(b(c), c)
-    arg_subs = tuple(arg.subs.items()) if isinstance(arg.subs, OrderedDict) else arg.subs
+    arg_subs = (
+        tuple(arg.subs.items()) if isinstance(arg.subs, OrderedDict) else arg.subs
+    )
     new_subs = subs + tuple((k, Subs(v, subs)) for k, v in arg_subs)
     return Subs(arg.arg, new_subs)
 
@@ -468,10 +586,14 @@ def unary_log_exp(op, arg):
     return arg.arg
 
 
-@normalize.register(Unary, ops.ReciprocalOp, Contraction[NullOp, ops.MulOp, frozenset, tuple])
+@normalize.register(
+    Unary, ops.ReciprocalOp, Contraction[NullOp, ops.MulOp, frozenset, tuple]
+)
 @normalize.register(Unary, ops.NegOp, Contraction[NullOp, ops.AddOp, frozenset, tuple])
 def unary_contract(op, arg):
-    return Contraction(arg.red_op, arg.bin_op, arg.reduced_vars, *(op(t) for t in arg.terms))
+    return Contraction(
+        arg.red_op, arg.bin_op, arg.reduced_vars, *(op(t) for t in arg.terms)
+    )
 
 
 BACKEND_TO_EINSUM_BACKEND = {
