@@ -1,6 +1,9 @@
 # Copyright Contributors to the Pyro project.
 # SPDX-License-Identifier: Apache-2.0
 
+from . import array, builtin, op, program, tracer
 from .array import *
 from .builtin import *
 from .op import *
+from .program import *
+from .tracer import *
